/*   =====================================================================
*    =     Copyright (C) 1989 by Racal-Milgo.  All Rights Reserved.      =
*    =        Confidential, Unpublished Property of Racal-Milgo.         =
*    =   Rights Reserved Under the Copyright Laws of the United States.  =
*    =====================================================================
*
*+=============================================================================
*
* Name:		FUEL_M
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		User Command Task
* Language:	C
* Index:	Display Graph of GM 1982 ECM Fuel Curves Varying MAP
*
* Function:
*
*	This function accepts a 9 by 9 array of hex fuel delivery codes
*	and produces a graph of a set of fuel curves on the PC screen.
*	Each fuel curve represents the fuel delivered at a given RPM
*	varied across 9 different MAP (manifold absolute pressure) readings.
*
*
* Synopsis:
*
*	FUEL_M  InputDataFile
*
*	InputDataFile -		The specification of a file containing 
*				hexadecimal fuel codes arranged in a 9 row 
*				by 9 column array, the rows containing fuel
*				delivery quantity codes for a single RPM 
*				across the range of MAP sensor readings.
*
*	EXIT STATUS -		Standard DOS exit status is provided.
*
*
* Additional Information:
*
*	An example input table is shown.  It is labeled here, although such 
*	labeling is not required.  FUEL_M will locate the first 9 lines of the
*	file which start with exactly 9 hex values separated by whitespace,
*	ignoring all else.
*
*	   2C 2E 30 36 3C 46 52 5F 68  | 600
*	   2C 31 3F 4C 5D 6E 7A 87 90  | 1200
*	   2D 33 4F 6F 8D 9C A3 A9 B0  | 1800
*	   2F 3F 64 86 9E A5 AC B2 B9  | 2400  Increasing
*	   31 4D 79 94 A5 AD B3 BB C1  | 3000     RPM
*	   3C 60 88 9E AD B5 BD C4 C9  | 3600      |
*	   4C 71 91 A7 B4 BE C5 CC D1  | 4200      V
*	   5E 88 A0 B0 BD C8 CF D5 DA  | 4800
*	   71 94 AC BB C8 D1 D8 DE E4  | 5400
*	   ____________________________|
*	   24 21 18 15 12 9  6  3  0
*	
*	   Increasing Manifold Pressure -->
*
*	After FUEL_M has been run, you may return to a DOS prompt by pressing
*	Return.
*
*
* Examples:
*
*	The following example shows how you would execute FUEL_M on the file
*	CHIP_17.TXT:
*
*		C> FUEL_M CHIP_17.TXT
*
*	
* Maintenance Information:
*
*	Notes:
*	------
*
*	This function was built to accept a pretty much free formatted text
*	file as input.  It ignores all but the hexadecimal values it locates.
*
*	A possible enhancement would be to become more sensitive to the table
*	labeling, and label the graph accordingly.  Presently, however, the
*	graph labeling is hard coded herein.
*
*
* 	Modification History:
*	---------------------
*
*	January 3, 1990		Noel Carboni
*
*	Initial conception, taken from TIMING.C.
*
*-=============================================================================
*/

/*
*==============================================================================
*
*  Defines.
*
*==============================================================================
*/
#define	MIN_MAP	 	0	/* Least (leftmost) MAP index displayed */
#define MAX_MAP		8	/* Greatest (rightmost) MAP index displayed */
#define MAP_PER_DIV	1	/* MAP indeces per dotted line division */

#define MIN_FUEL	0x00	/* Least (bottom) allowable fuel code */
#define MAX_FUEL 	0x100	/* Greatest (top) allowable fuel code */
#define FUEL_PER_DIV	0x10	/* Fuel codes per dotted line division */

#define BOTTOM_BORDER	0	/* # of blank divisions to leave on bottom */
#define TOP_BORDER	0	/* # of blank divisions to leave on top */
#define LEFT_BORDER	0 	/* # of blank divisions to leave on left */
#define RIGHT_BORDER	0	/* # of blank divisions to leave on right */

 
/*
*==============================================================================
*
*  Includes.
*
*==============================================================================
*/
#include "stdio.h"		/* Standard input/output definitions */
#include "stdlib.h"		/* Standard library function prototypes */
#include "string.h"		/* String library function prototypes */
#include "graph.h"		/* Graphics library definitions */
#include "errno.h"		/* System error message definitions */

/*
*==============================================================================
*
*  Macro definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External declarations.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*
*  Local static variable definitions.
*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*/
struct videoconfig config;	/* Holds the video configuration */

char	input_file_name[80];	/* User specified input file name */

FILE	*in_file;		/* Input file stream pointer */

char	input_line[256];	/* Input line from file */

char	input_template[] = 	/* scanf template for input translation */
	"%2X %2X %2X %2X %2X %2X %2X %2X %2X %2X ";

int	fuel[9][9];		/* Input fuel codes from file */


/*
*==============================================================================
*
*  Local function prototypes.
*
*==============================================================================
*/
short y(int fuel_code);
short x(int map_index);
void error_exit(int status, char *message);

/*
*==============================================================================
*
*  Start of task.
*
*==============================================================================
*/
void main(int argc, char **argv, char **envp)
{
    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Local variable definitions
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    int i;			/* Loop control variable */

    int	rpm;			/* Fuel code table row index */
    int	mapval;			/* Fuel code table column index */
    int dummy;			/* Used to test for exactly 9 hex codes */

    /*
    *==========================================================================
    *
    *  Get input file information from the command line.
    *
    *==========================================================================
    *
    *  Validate and copy the input parameters.
    */
    envp = envp;	/* Prevents warning message */

    if (argc != 2)
       error_exit(EINVAL, "Required parameter is Inputfile");

    strncpy(input_file_name, argv[1], 80);
    
    /*
    *  Attempt to open the files specified.
    */
    in_file = fopen(input_file_name, "rt");
    if (in_file == NULL)
	error_exit(ENOENT, "Could not open input file");

    /*
    *==========================================================================
    *
    *  Read the input file, attempting to locate a total 9 lines, each 
    *   containing exactly 9 hexadecimal fuel code values.  If 9 lines
    *   cannot be found prior to reaching the end of the file, we will exit
    *   with an error.  If all 9 lines are found, we will close the file and
    *   move on.
    *
    *==========================================================================
    */
    rpm = 0;
    
    while (rpm < 9)
    {
	if (fgets(input_line, 255, in_file) == NULL)
	    error_exit(ENOENT, 
	    "Could not locate all fuel data in input file");
	
	if (sscanf(input_line, input_template,
	&fuel[rpm][0],  &fuel[rpm][1],  &fuel[rpm][2],  &fuel[rpm][3],
	&fuel[rpm][4],  &fuel[rpm][5],  &fuel[rpm][6],  &fuel[rpm][7],
	&fuel[rpm][8],  dummy ) == 9)
	    rpm++;
    }

    fclose(in_file);

    /*
    *==========================================================================
    *
    *  Plot the data on a graph.
    *
    *==========================================================================
    */
    _setvideomode(_HRESBW);   	/* 320 by 200 graphics, no colors */
    _getvideoconfig(&config);	/* Get graphics configuration */

    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Draw bounding box and label it.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  First, the box.
    */
    _setcolor(15);			/* Medium grey */

    _moveto(x(MIN_MAP), y(MAX_FUEL));	/* Upper left corner */
    _lineto(x(MIN_MAP), y(MIN_FUEL));	/* Left side */
    _lineto(x(MAX_MAP), y(MIN_FUEL));	/* Bottom */
    _lineto(x(MAX_MAP), y(MAX_FUEL));	/* Right side */
    _lineto(x(MIN_MAP), y(MAX_FUEL));	/* Top */

    /*
    *  Draw dotted horizontal reference lines for each 10 hex of fuel.
    */
    _setlinestyle(0x0101);		/* Dotted line */
    _setcolor(7);			/* Dim grey */

    for (i = MIN_FUEL+FUEL_PER_DIV; i <= MAX_FUEL-FUEL_PER_DIV; i += FUEL_PER_DIV)
    {
	_moveto(x(MIN_MAP), y(i));
	_lineto(x(MAX_MAP), y(i));
    }

    /*
    *  Output horizontal reference labeling.
    */

    /* -- Not implemented yet -- */




    /*
    *  Draw dotted vertical reference lines for each throttle position.
    */
    _setlinestyle(0x1111);		/* Dotted line */

    for (i = MIN_MAP+MAP_PER_DIV; i <= MAX_MAP-MAP_PER_DIV; i += MAP_PER_DIV)
    {
	_moveto(x(i), y(MIN_FUEL));
	_lineto(x(i), y(MAX_FUEL));
    }
    
    /*
    *  Output vertical reference labeling.
    */

    /* -- Not implemented yet -- */


    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Plot the data on the graph.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    _setlinestyle(0xFFFF);		/* Solid line */
    for (rpm = 0; rpm < 9; rpm++)
    {
	_setcolor(rpm+1);			/* A color for each position */
	_moveto(x(MIN_MAP), y(fuel[rpm][MIN_MAP]));
	for (mapval = MIN_MAP + 1; mapval <= MAX_MAP; mapval++)
	    _lineto(x(mapval), y(fuel[rpm][mapval]));

    }


    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Await a Carriage Return character, then return screen to original format.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    while (getchar() != '\n');		/* Await a Carriage Return from user */

    fcloseall();			/* Close all open files */
    _setvideomode(_MRES16COLOR);
    _setvideomode(_TEXTC80);		/* 80 x 25 text, 16 color */


    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Exit, returning the status of the operation to DOS.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    exit(0);		

}
/* End of function main() */
                       
/*
*+=============================================================================
*
* Name:		x
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Calculate a column screen coordinate given an rpm code.
*
* Function:
*
*	This function accepts an rpm code and returns the column screen
*	coordinate.
*
*
* Synopsis:
*
*	short x(int map_index)
*
*	map_index -	The map index to be converted.  Values from MIN_MAP
*			through MAX_MAP are acceptable.
*
*	RETURN -	The column (x) screen coordinate.
*
*
*-=============================================================================
*/
short x(int map_index)
{
    #define PIXELS_PER_H_DIV  ((config.numxpixels-1)		      \
        / (((MAX_MAP - MIN_MAP) / MAP_PER_DIV) + LEFT_BORDER + RIGHT_BORDER))

    #define PIXELS_ON_LEFT  (LEFT_BORDER * PIXELS_PER_H_DIV)
    #define PIXELS_ON_RIGHT  (RIGHT_BORDER * PIXELS_PER_H_DIV)

    return((short) PIXELS_ON_LEFT + ((map_index - MIN_MAP) * PIXELS_PER_H_DIV
        / MAP_PER_DIV));

}
/* End of function x() */

/*
*+=============================================================================
*
* Name:		y
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Calculate a row screen coordinate given a fuel code.
*
* Function:
*
*	This function accepts a fuel code and returns the row screen
*	coordinate.
*
*
* Synopsis:
*
*	short y(int fuel_code)
*
*	fuel_code -	The fuel code to be converted.  Values from MIN_FUEL
*			through MAX_FUEL are acceptable.
*
*	RETURN -	The row (y) screen coordinate.
*
*
*-=============================================================================
*/
short y(int fuel_code)
{
    #define PIXELS_ON_TOP	(TOP_BORDER * FUEL_PER_DIV)
    #define PIXELS_ON_BOTTOM	(BOTTOM_BORDER * FUEL_PER_DIV)

    return((short) (config.numypixels - 1 - PIXELS_ON_BOTTOM -
    ((long) (fuel_code - MIN_FUEL)
    * (config.numypixels - 1 - PIXELS_ON_TOP - PIXELS_ON_BOTTOM)
    / (MAX_FUEL - MIN_FUEL))));
}
/* End of function y() */

/*
*+=============================================================================
*
* Name:		error_exit
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Display error message and exit.
*
* Function:
*
*	This function will display an error message text string and exit the
*	task, returning a specified standard error message code to DOS.
*
*
* Synopsis:
*
*	void error_exit(int status, char *message)
*
*	status -	Standard error code, as defined in errno.h.
*
*	message - 	Pointer to a null terminated string containing the
*			error message to be displayed.
*
*	RETURN -	This function does not return.
*
*
*-=============================================================================
*/
void error_exit(int status, char *message)
{
    errno = status;	/* Load status into standard global error variable */
    perror(message);	/* Display the error message */
    exit (status);	/* Get the flock out of here */

}
/* End of function error_exit() */

/*
*==============================================================================
*
*  End of file.
*
*==============================================================================
*/
