/*   =====================================================================
*    =     Copyright (C) 1989 by Racal-Milgo.  All Rights Reserved.      =
*    =        Confidential, Unpublished Property of Racal-Milgo.         =
*    =   Rights Reserved Under the Copyright Laws of the United States.  =
*    =====================================================================
*
*+=============================================================================
*
* Name:		SPARKHEX
* Ident:	V01.00 (Non-SCCS)
* Type:		User Command Task
* Language:	C
* Index:	Convert text file containing timing info to hex
*
* Function:
*
*	This function combines a hex file containing 1982 Corvette ECM 
*	data tables with spark data from a text file to produce another
*	hex file.
*	
*
* Synopsis:
*
*	SPARKHEX   HexFile  TextFile  [> OutFile]
*
*	HexFile -		The specification of an Intel format hex file 
*				to be used as input and containing ECM data 
*				from address 0000 hex to address 0FFF hex.
*
*	TextFile -		The specification of a text file containing 
*				hexadecimal timing codes arranged in a 
*				14 by 15 array.
*
*	> OutFile -		The output hex file is generated to Standard
*				Output, which may be redirected to OutFile
*				through the use of the DOS output redirection
*				operator (>).
*
*	EXIT STATUS -		Standard DOS exit status is provided.  Error
*				level 0 is returned if operation is successful.
*
*
* Additional Information:
*
*	SPARKHEX expects the text file to contain a 14 by 15 array of hex spark 
*	advance codes.
*
*	SPARKHEX overlays the hex file provided with the data from the text 
*	file, then duplicates the 1K bytes from 0C00 hex through 0FFF hex 
*	into 0000 hex, 0400 hex, and 0800 hex.  This will ensure that the
*	right data will be read even if the top two address lines float.
*
*	An example input file is shown.  It contains miscellaneous text and
*	labeling, although such information is not required.  SPARKHEX will 
*	locate the first 14 lines of the file which start with exactly 15 hex 
*	values separated by whitespace and assume this is timing information.
*
*
*	    CUSTOM_CHIP_21 spark advance curves 1/19/90.
*
*	    Spark advance table at address 0C1A:
*
*	    45 45 45 40 3C 39 35 32 2F 2D 2A 28 26 25 24  | 0
*	    30 30 45 40 3C 39 35 32 2F 2D 2A 28 26 25 24  | 300
*	    24 24 42 3E 3A 36 33 31 2E 2C 2A 28 26 25 24  | 600
*	    24 24 41 3D 39 35 32 30 2E 2C 2A 28 26 25 24  | 900
*	    25 25 40 3C 38 34 31 2F 2D 2B 29 28 27 26 25  | 1200
*	    26 26 40 3C 38 35 32 2F 2D 2B 2A 29 28 27 26  | 1500    Increasing
*	    3A 3A 41 3D 3A 36 34 31 2F 2D 2C 2B 2A 29 28  | 1800        RPM
*	    4D 49 44 40 3C 3A 37 35 33 31 2F 2E 2D 2C 2B  | 2100         |
*	    56 51 4B 47 43 40 3D 3B 38 36 34 33 31 30 2F  | 2400         V
*	    62 5C 55 4F 4A 47 44 41 3E 3C 3A 39 37 36 35  | 2700
*	    6B 65 5F 58 53 4F 4C 49 46 44 42 41 3F 3E 3D  | 3000
*	    71 6B 65 5F 5A 57 55 52 4F 4D 4B 4A 49 48 47  | 3300
*	    73 6E 69 64 60 5D 5A 58 56 54 52 51 4F 4E 4D  | 3600
*	    74 70 6B 67 64 61 5E 5C 59 57 56 54 52 51 50  | 3900
*	    ______________________________________________|
*	    1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
*		   Increasing manifold pressure -->
*
*
*
* Examples:
*
*	The following example shows how you would execute SPARKHEX on the file
*	CHIP_17.TXT to produce the file OVLAY_17.HEX
*
*		C> SPARKHEX CHIP_17.TXT > OVLAY_17.HEX
*
*	
* Maintenance Information:
*
*	Notes:
*	------
*
*	This function was built to accept a pretty much free formatted text
*	file as input.  It ignores all but the hexadecimal values it locates.
*
*
* 	Modification History:
*	---------------------
*
*	January 23, 1990	Noel Carboni
*
*	Initial conception, taken from TXTHEX.C.  Removed fuel table
*	processing, since the F.I.R.S.T. TPI system is now performing the
*	fuel management.
*
*-=============================================================================
*/

/*
*==============================================================================
*
*  Defines.
*
*==============================================================================
*/
#define	DATA_RECORD  	   0	/* Intel data record */
#define	EOF_RECORD   	   1	/* Intel end of file record */
#define	EXT_ADDR_RECORD	   2	/* Intel extended address record */
#define	START_ADDR_RECORD  3	/* Intel start address record */
 
/*
*==============================================================================
*
*  Includes.
*
*==============================================================================
*/
#include "stdio.h"		/* Standard input/output definitions */
#include "stdlib.h"		/* Standard library function prototypes */
#include "string.h"		/* String library function prototypes */
#include "errno.h"		/* System error message definitions */

/*
*==============================================================================
*
*  Macro definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External declarations.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*
*  Local static variable definitions.
*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*/
char	hex_file_name[80];	/* User specified input hex file name */
char	text_file_name[80];	/* User specified input text file name */

FILE	*hex_file;		/* Input hex file stream pointer */
FILE	*text_file;		/* Input text file stream pointer */

char	input_line[256];	/* Input line from file */

char	hex_rec_template[] = 	/* scanf template for input translation */
	    ":%2X%4X%2X";
char	hex_byte_template[] = 	/* scanf template for input translation */
	    "%2X";

char	text_template[] = 	/* scanf template for input translation */
	    "%2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X ";

unsigned char	prom[4096];	/* Array of PROM data */

unsigned int	code[15];	/* One line of converted input codes. */

unsigned int	adv[14][15];	/* Input advance codes from file */

/*
*==============================================================================
*
*  Local function prototypes.
*
*==============================================================================
*/
void error_exit(int status, char *message);

/*
*==============================================================================
*
*  Start of task.
*
*==============================================================================
*/
void main(int argc, char **argv, char **envp)
{
    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Local variable definitions
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    unsigned int i,j;		/* Loop control variables */

    char *input_ptr;		/* Pointer within hex record to data bytes */
    unsigned int num_fields;	/* Number of hex fields xlated from input */
    unsigned int record_type;	/* Intel hex record type from input */
    unsigned int data_byte;	/* Data byte from hex record */

    unsigned int dummy;		/* To determine if there are too many codes */

    int	timing_row;		/* Timing code table row index */

    unsigned int num_bytes;	/* Number of data bytes in output string */
    unsigned int address;	/* Address of the data bytes */
    unsigned int checksum;	/* Checksum of output line */

    /*
    *==========================================================================
    *
    *  Get input file information from the command line.
    *
    *==========================================================================
    *
    *  Validate and copy the input parameters.
    */
    envp = envp;	/* Prevents unreferenced variable warning message */

    if (argc != 3)
        error_exit(EINVAL, "Format is SPARKHEX HexFile TextFile [> OutFile]");

    strncpy(hex_file_name, argv[1], 80);
    strncpy(text_file_name, argv[2], 80);
    
    /*
    *  Attempt to open the specified input hex file.
    */
    hex_file = fopen(hex_file_name, "rt");
    if (hex_file == NULL)
	error_exit(ENOENT, "Could not open input hex file");

    /*
    *  Attempt to open the specified input text file.
    */
    text_file = fopen(text_file_name, "rt");
    if (text_file == NULL)
	error_exit(ENOENT, "Could not open input text file");

    /*
    *==========================================================================
    *
    *  Read the hex file and fill in the prom array.
    *
    *==========================================================================
    */
    while (fgets(input_line, 255, hex_file) != NULL)
    {
	num_fields = sscanf(input_line, hex_rec_template, 
	    &num_bytes, 
	    &address,
	    &record_type);

	/*
	*  If the number of bytes, address, and record type were successfully 
	*   found in the line, and this is a data record, decode the hex data 
	*   values.  Note:  The checksum is ignored.
	*/
	if (num_fields == 3 && record_type == DATA_RECORD)
	{
	    input_ptr = &input_line[9];
	    for (i=0; i<num_bytes; i++)
	    {
		if (sscanf(input_ptr, hex_byte_template, &data_byte) == 1)
		{
		    prom[address++] = (unsigned char) data_byte;
		    input_ptr += 2;
		}
		else
		    error_exit(ERANGE, "Invalid value in hex file");
	    }
	}
    }
    fclose(hex_file);

    /*
    *==========================================================================
    *
    *  Read the input file, attempting to locate the timing values.  As they 
    *   are located, fill in the row of the timing array.
    *
    *==========================================================================
    */
    timing_row = 0;
    
    while (timing_row < 14)
    {
	if (fgets(input_line, 255, text_file) == NULL)
	    error_exit(ENFILE, 
	    "Could not locate all timing data in text file");
	
	num_fields = sscanf(input_line, text_template,
	    &code[0],  &code[1],  &code[2],  &code[3],
	    &code[4],  &code[5],  &code[6],  &code[7],
	    &code[8],  &code[9],  &code[10], &code[11],
	    &code[12], &code[13], &code[14], &dummy);

	/*
	*  If there are exactly 15 codes we have a line of timing data.
	*/
	if (num_fields == 15)
	{
	    for (i=0; i<15; i++)
		adv[timing_row][i] = code[i];

	    timing_row++;
	}
    }

    fclose(text_file);

    /*
    *==========================================================================
    *
    *  Generate the output data.
    *
    *  A line of Intel hex output has the following format (spaces added here
    *   for clarity only):
    *
    *	    :NN AAAA 00 NN_data_bytes CC
    *
    *	    Colon precedes every line.
    *	    NN is the number of data bytes contained in the message in hex.
    *	    AAAA is the absolute starting address.
    *	    00 separates the address and data fields.
    *	    Data bytes in hex follow.
    *	    The last thing on the line is the 2 digit negated checksum in hex.
    *
    *  The last line in the file must be:
    *
    *       :00000001FF
    *
    *==========================================================================
    *
    *  First, copy the timing array to the appropriate place in the 0C00 
    *   to 0FFF section of the PROM array, then copy that section to 0000, 
    *   0400, and 0800.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    address = 0x0C1A;
    for (i=0; i<14; i++)
	for (j=0; j<15; j++)
	    prom[address++] = (unsigned char) adv[i][j];

    for (address=0x0000; address<0x0400; address++)
    {
	prom[address + 0x0000] = prom[address + 0x0C00];
	prom[address + 0x0400] = prom[address + 0x0C00];
	prom[address + 0x0800] = prom[address + 0x0C00];
    }
    
    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Output the entire prom array in Intel hex.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    for (address=0x0000; address<0x1000; address+=16)
    {
	checksum = 16 + (address & 0xFF) + ((address >> 8) & 0xFF);
	for (i=0; i<16; i++)
	    checksum += prom[address + i];

	printf(
	    ":%2.2X%4.4X00%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X\n",
	    0xFF & 16,
	    address,
	    0xFF & prom[address + 0],
	    0xFF & prom[address + 1],
	    0xFF & prom[address + 2],
	    0xFF & prom[address + 3],
	    0xFF & prom[address + 4],
	    0xFF & prom[address + 5],
	    0xFF & prom[address + 6],
	    0xFF & prom[address + 7],
	    0xFF & prom[address + 8],
	    0xFF & prom[address + 9],
	    0xFF & prom[address + 10],
	    0xFF & prom[address + 11],
	    0xFF & prom[address + 12],
	    0xFF & prom[address + 13],
	    0xFF & prom[address + 14],
	    0xFF & prom[address + 15],
	    0xFF & -(signed int)checksum);
    }

    /*
    *  Output the termination line.
    */
    printf(":00000001FF\n");

    fcloseall();			/* Close all open files */

    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Exit, returning success status to DOS.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    exit(0);		

}
/* End of function main() */
                       
/*
*+=============================================================================
*
* Name:		error_exit
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Display error message and exit.
*
* Function:
*
*	This function will display an error message text string and exit the
*	task, returning a specified standard error message code to DOS.
*
*
* Synopsis:
*
*	void error_exit(int status, char *message)
*
*	status -	Standard error code, as defined in errno.h.
*
*	message - 	Pointer to a null terminated string containing the
*			error message to be displayed.
*
*	RETURN -	This function does not return.
*
*
*-=============================================================================
*/
void error_exit(int status, char *message)
{
    errno = status;	/* Load status into standard global error variable */
    perror(message);	/* Display the error message */
    exit (status);	/* Get the flock out of here */

}
/* End of function error_exit() */

/*
*==============================================================================
*
*  End of file.
*
*==============================================================================
*/
