/*   =====================================================================
*    =     Copyright (C) 1989 by Racal-Milgo.  All Rights Reserved.      =
*    =        Confidential, Unpublished Property of Racal-Milgo.         =
*    =   Rights Reserved Under the Copyright Laws of the United States.  =
*    =====================================================================
*
*+=============================================================================
*
* Name:		TIMING
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		User Command Task
* Language:	C
* Index:	Display Graph of GM 1982 ECM Timing Curves
*
* Function:
*
*	This function accepts a 14 by 15 array of hex spark advance codes
*	and produces a labeled graph of a set of timing curves on the PC screen.
*
*
* Synopsis:
*
*	TIMING  InputDataFile
*
*	InputDataFile -		The specification of a file containing 
*				hexadecimal timing codes arranged in a 14 row 
*				by 15 column array, the rows containing spark 
*				advance codes for a single RPM across the 
*				range of throttle positions.
*
*	EXIT STATUS -	Standard DOS exit status is provided.
*
*
* Additional Information:
*
*	An example input table is shown.  It is labeled here, although such 
*	labeling is not required.  TIMING will locate the first 14 lines of
*	the file which start with exactly 15 hex values separated by 
*	whitespace, ignoring all else.
*
*	    39 39 39 39 39 39 39 39 39 39 22 22 22 22 22  | 400
*	    39 39 39 39 39 39 39 39 39 39 25 25 25 25 25  | 800
*	    2A 2A 39 39 39 39 39 39 39 39 2B 28 28 28 28  | 1200
*	    2A 2A 47 47 47 47 47 47 47 47 39 33 2B 2B 2B  | 1600
*	    32 32 4E 4E 4E 4E 4E 4E 4E 4E 40 34 2E 2E 2E  | 2000
*	    35 35 55 55 55 55 55 55 55 51 4B 36 36 36 36  | 2400    Increasing
*	    38 38 69 69 69 69 69 69 69 58 55 53 4E 48 43  | 2800        RPM
*	    3F 3F 6F 6F 6F 6F 6F 6F 6F 5F 5C 59 54 50 4C  | 3200         |
*	    46 46 75 75 75 75 75 75 75 64 61 5F 5B 56 52  | 3600         V
*	    64 64 75 75 75 75 75 75 75 69 66 64 5F 5A 56  | 4000
*	    77 77 77 77 77 77 77 77 77 6D 6A 68 63 5E 59  | 4400
*	    7A 7A 7A 7A 7A 7A 7A 7A 7A 70 6D 6A 65 60 5C  | 4800
*	    7D 7D 7D 7D 7D 77 72 72 72 71 6E 6C 67 62 5D  | 5200
*	    81 81 81 81 81 7A 79 78 75 72 6F 6D 67 62 5D  | 5600
*	    ______________________________________________|
*	    1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
*	    		Increasing throttle -->
*
*
*	After TIMING has been run, you may return to a DOS prompt by pressing
*	Return.
*
*
* Examples:
*
*	The following example shows how you would execute TIMING on the file
*	CC14.TXT:
*
*		C> TIMING CC14.TXT
*
*	
* Maintenance Information:
*
*	Notes:
*	------
*
*	This function was built to accept a pretty much free formatted text
*	file as input.  It ignores all but the hexadecimal values it locates.
*
*	A possible enhancement would be to become more sensitive to the table
*	labeling, and label the graph accordingly.  Presently, however, the
*	graph labeling is hard coded herein.
*
*
* 	Modification History:
*	---------------------
*
*	September 20, 1989		Noel Carboni
*
*	Initial conception to partially automate the process of making fine
*	adjustments to the tables within the GM 1989 ECM control PROM.
*
*	January 25, 1990		Noel Carboni
*
*	Removed requirement for an output file specification.
*
*-=============================================================================
*/

/*
*==============================================================================
*
*  Defines.
*
*==============================================================================
*/
#define	MIN_RPM	 	0	/* Least (leftmost) RPM index displayed */
#define MAX_RPM		13	/* Greatest (rightmost) RPM index displayed */
#define RPM_PER_DIV	1	/* RPM indeces per dotted line division */

#define MIN_ADV		32	/* Least (bottom) allowable advance code */
#define MAX_ADV 	128	/* Greatest (top) allowable advance code */
#define ADV_PER_DIV	16	/* Advance codes per dotted line division */

#define BOTTOM_BORDER	0	/* # of blank divisions to leave on bottom */
#define TOP_BORDER	0	/* # of blank divisions to leave on top */
#define LEFT_BORDER	0 	/* # of blank divisions to leave on left */
#define RIGHT_BORDER	0	/* # of blank divisions to leave on right */

 
/*
*==============================================================================
*
*  Includes.
*
*==============================================================================
*/
#include "stdio.h"		/* Standard input/output definitions */
#include "stdlib.h"		/* Standard library function prototypes */
#include "string.h"		/* String library function prototypes */
#include "graph.h"		/* Graphics library definitions */
#include "errno.h"		/* System error message definitions */

/*
*==============================================================================
*
*  Macro definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External declarations.
*
*==============================================================================
*/
/* -- None -- */

/*
*==============================================================================
*
*  External definitions.
*
*==============================================================================
*/
/* -- None -- */

/*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*
*  Local static variable definitions.
*
* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
*/
struct videoconfig config;	/* Holds the video configuration */

char	buffer[16];		/* Output text */

char	input_file_name[80];	/* User specified input file name */

FILE	*in_file;		/* Input file stream pointer */

char	input_line[256];	/* Input line from file */

char	input_template[] = 	/* scanf template for input translation */
	"%2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X %2X ";

int	adv[14][15];		/* Input advance codes from file */


/*
*==============================================================================
*
*  Local function prototypes.
*
*==============================================================================
*/
short y(int timing_code);
short x(int rpm_index);
void error_exit(int status, char *message);

/*
*==============================================================================
*
*  Start of task.
*
*==============================================================================
*/
void main(int argc, char **argv, char **envp)
{
    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Local variable definitions
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    int i;			/* Loop control variable */

    int	rpm;			/* Advance code table row index */
    int	posn;			/* Advance code table column index */

    /*
    *==========================================================================
    *
    *  Get input file information from the command line.
    *
    *==========================================================================
    *
    *  Validate and copy the input parameters.
    */
    envp = envp;	/* Prevents warning message */

    if (argc != 2)
       error_exit(EINVAL, "Usage is TIMING TextFile");

    strncpy(input_file_name, argv[1], 80);
    
    /*
    *  Attempt to open the files specified.
    */
    in_file = fopen(input_file_name, "rt");
    if (in_file == NULL)
	error_exit(ENOENT, "Could not open input file");

    /*
    *==========================================================================
    *
    *  Read the input file, attempting to locate a total 14 lines, each 
    *   containing exactly 15 hexadecimal advance code values.  If 14 lines
    *   cannot be found prior to reaching the end of the file, we will exit
    *   with an error.  If all 14 lines are found, we will close the file and
    *   move on.
    *
    *==========================================================================
    */
    rpm = 0;
    
    while (rpm < 14)
    {
	if (fgets(input_line, 255, in_file) == NULL)
	    error_exit(ENOENT, 
	    "Could not locate all advance data in input file");
	
	if (sscanf(input_line, input_template,
	&adv[rpm][0],  &adv[rpm][1],  &adv[rpm][2],  &adv[rpm][3],
	&adv[rpm][4],  &adv[rpm][5],  &adv[rpm][6],  &adv[rpm][7],
	&adv[rpm][8],  &adv[rpm][9],  &adv[rpm][10], &adv[rpm][11],
	&adv[rpm][12], &adv[rpm][13], &adv[rpm][14])
	== 15)
	    rpm++;
    }

    fclose(in_file);

    /*
    *==========================================================================
    *
    *  Plot the data on a graph.
    *
    *==========================================================================
    */
    _setvideomode(_HRESBW);   	/* 320 by 200 graphics, no colors */
    _getvideoconfig(&config);	/* Get graphics configuration */

    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Draw bounding box and label it.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  First, the box.
    */
    _setcolor(15);			/* Medium grey */

    _moveto(x(MIN_RPM), y(MAX_ADV));	/* Upper left corner */
    _lineto(x(MIN_RPM), y(MIN_ADV));	/* Left side */
    _lineto(x(MAX_RPM), y(MIN_ADV));	/* Bottom */
    _lineto(x(MAX_RPM), y(MAX_ADV));	/* Right side */
    _lineto(x(MIN_RPM), y(MAX_ADV));	/* Top */

    /*
    *  Draw dotted horizontal reference lines for each 10 hex of advance.
    */
    _setlinestyle(0x0101);		/* Dotted line */
    _setcolor(7);			/* Dim grey */

    for (i = MIN_ADV+ADV_PER_DIV; i <= MAX_ADV-ADV_PER_DIV; i += ADV_PER_DIV)
    {
	_moveto(x(MIN_RPM), y(i));
	_lineto(x(MAX_RPM), y(i));
    }

    /*
    *  Output horizontal reference labeling.
    */
    /* -- Not implemented -- */




    /*
    *  Draw dotted vertical reference lines for each throttle position.
    */
    _setlinestyle(0x1111);		/* Dotted line */

    for (i = MIN_RPM+RPM_PER_DIV; i <= MAX_RPM-RPM_PER_DIV; i += RPM_PER_DIV)
    {
	_moveto(x(i), y(MIN_ADV));
	_lineto(x(i), y(MAX_ADV));
    }
    
    /*
    *  Output vertical reference labeling.
    */
    /* -- Not implemented -- */


    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Plot the advance data on the graph.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    _setlinestyle(0xFFFF);		/* Solid line */
    for (posn = 0; posn < 15; posn++)
    {
	_setcolor(posn+1);			/* A color for each position */
	_moveto(x(MIN_RPM), y(adv[MIN_RPM][posn]));
	for (rpm = MIN_RPM + 1; rpm <= MAX_RPM; rpm++)
	    _lineto(x(rpm), y(adv[rpm][posn]));

    }

    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Await a Carriage Return character, then return screen to original format.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    while(getchar() != '\n');		/* Await a carriage return from user */

    fcloseall();			/* Close all open files */
    _setvideomode(_MRES16COLOR);
    _setvideomode(_TEXTC80);		/* 80 x 25 text, 16 color */


    /*
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    *
    *  Exit, returning the status of the operation to DOS.
    *
    * = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
    */
    exit(0);		

}
/* End of function main() */
                       
/*
*+=============================================================================
*
* Name:		x
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Calculate a column screen coordinate given an rpm code.
*
* Function:
*
*	This function accepts an rpm code and returns the column screen
*	coordinate.
*
*
* Synopsis:
*
*	short x(int rpm_code)
*
*	rpm_code -	The rpm code to be converted.  Values from MIN_RPM
*			through MAX_RPM are acceptable.
*
*	RETURN -	The column (x) screen coordinate.
*
*
*-=============================================================================
*/
short x(int rpm_index)
{
    #define PIXELS_PER_H_DIV  (config.numxpixels 		      \
        / (((MAX_RPM - MIN_RPM) / RPM_PER_DIV) + LEFT_BORDER + RIGHT_BORDER))

    #define PIXELS_ON_LEFT  (LEFT_BORDER * PIXELS_PER_H_DIV)
    #define PIXELS_ON_RIGHT  (RIGHT_BORDER * PIXELS_PER_H_DIV)

    return((short) PIXELS_ON_LEFT + ((rpm_index - MIN_RPM) * PIXELS_PER_H_DIV
        / RPM_PER_DIV));

}
/* End of function x() */

/*
*+=============================================================================
*
* Name:		y
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Calculate a row screen coordinate given a timing code.
*
* Function:
*
*	This function accepts a timing code and returns the row screen
*	coordinate.
*
*
* Synopsis:
*
*	short y(int timing_code)
*
*	timing_code -	The timing code to be converted.  Values from MIN_ADV
*			through MAX_ADV are acceptable.
*
*	RETURN -	The row (y) screen coordinate.
*
*
*-=============================================================================
*/
short y(int timing_code)
{
    #define PIXELS_ON_TOP	(TOP_BORDER * ADV_PER_DIV)
    #define PIXELS_ON_BOTTOM	(BOTTOM_BORDER * ADV_PER_DIV)

    return((short) (config.numypixels - 1 - PIXELS_ON_BOTTOM -
    ((long) (timing_code - MIN_ADV)
    * (config.numypixels - 1 - PIXELS_ON_TOP - PIXELS_ON_BOTTOM)
    / (MAX_ADV - MIN_ADV))));
}
/* End of function y() */

/*
*+=============================================================================
*
* Name:		error_exit
* Ident:	This SCCS version %I% of %M% was created on %G%.
* Type:		Local Function/Subroutine
* Language:	C
* Index:	Display error message and exit.
*
* Function:
*
*	This function will display an error message text string and exit the
*	task, returning a specified standard error message code to DOS.
*
*
* Synopsis:
*
*	void error_exit(int status, char *message)
*
*	status -	Standard error code, as defined in errno.h.
*
*	message - 	Pointer to a null terminated string containing the
*			error message to be displayed.
*
*	RETURN -	This function does not return.
*
*
*-=============================================================================
*/
void error_exit(int status, char *message)
{
    errno = status;	/* Load status into standard global error variable */
    perror(message);	/* Display the error message */
    exit (status);	/* Get the flock out of here */

}
/* End of function error_exit() */

/*
*==============================================================================
*
*  End of file.
*
*==============================================================================
*/
